"""Dashboard views - compact and detail modes."""

from raxe.cli.dashboard.views.base_view import BaseView
from raxe.cli.dashboard.views.compact_view import CompactView
from raxe.cli.dashboard.views.detail_view import DetailView

__all__ = ["BaseView", "CompactView", "DetailView"]
